const { Client } = require("@elastic/elasticsearch");
import { O_DIRECT } from "constants";
import { HttpService } from "./http.service";

export class ElasticSearchService {
  client: any;
  httpService: HttpService;
  constructor(host: string = "http://localhost:9200", version: string = "7.6") {
    this.client = new Client({ node: host });
    this.httpService = new HttpService(host);
  }

  async addItem(index: string, obj: any) {
    const { body } = await this.client.index({
      index: index,
      body: obj,
    });
    this.refreshIndex(index);
    return body._id;
  }

  async addMultipleItems(index: string, objArray: Array<any>) {
    console.log('123')
    var idsArr = [];
    for (const obj of objArray) {
      const id = await this.addItem(index, obj);
      idsArr.push(id);
    }
    return idsArr;
  }

  async deleteDocumentByElasticId(index: string, id: string) {
    const ans = await this.httpService.deleteByElasticIp(index, id);
    console.log("ans: ", ans);
    return ans;
  }
  async deleteByField(index: string, field: string, value: string) {
    const ans = await this.httpService.deleteByField(index, field, value);
    console.log("ans: ", ans);
    return ans;
  }
  async deleteByMultipleFields(index: string, deleteObj: any) { 
    const query = await this.buildQuery(deleteObj);
    const ans = await this.httpService.deleteByMultipleFields(index, query);
    console.log("ans: ", ans);
    return ans;
  }

  async searchByField(index: string, field: string, value: string) {
    const { body } = await this.client.search({
      index: index,
      body: {
        query: {
          match: { [field]: value },
        },
      },
    });
    const result = await this.desrilize(body.hits.hits);
    return result;
  }
  async search(index: string, searchObj: any) {
    const query = await this.buildQuery(searchObj);
    const { body } = await this.client.search({
      index: index,
      body: {
        query: {
          bool: {
            must: query,
          },
        },
      },
    });
    const result = await this.desrilize(body.hits.hits);
    return result;
  }
  async getByElasticId(index: string, id: string) {
    // const query = await this.buildQuery(searchObj);
    const result = await this.httpService.getByElasticId(index,id)
    const result2 = await this.desrilize(result);
    return result2;
    // return result;
  }

  async refreshIndex(index: string) {
    await this.client.indices.refresh({ index: index });
  }
  async buildQuery(obj: any) {
    var query = [];
    for (const key in obj) {
      query.push({
        match: {
          [key]: obj[key],
        },
      });
    }
    console.log("query: ", query);
    return query;
  }

  async desrilize(obj: any) {
    if(!Array.isArray(obj)){
      obj = [obj]
    }
    const newObj = obj.map((obj: any) => {
      return obj._source;
    });
    return newObj;
  }
}
